@@ -18,7 +18,11 @@
 #include <KConfigWatcher>
 #include <KFileUtils>
 #include <KPluginMetaData>
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 72)
 #include <KServiceTypeTrader>
+#else
+#define KSERVICE_BUILD_DEPRECATED_SINCE(a, b) 0
+#endif
 #include <KSharedConfig>
 
 #include "config.h"
@@ -30,9 +34,12 @@
 #include <ThreadWeaver/Queue>
 #include <ThreadWeaver/Thread>
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 65)
 #include <plasma/version.h>
+#endif
 
 #include "dbusrunner_p.h"
+#include "kpluginmetadata_utils_p.h"
 #include "krunner_debug.h"
 #include "querymatch.h"
 #include "runnerjobs_p.h"
@@ -41,7 +48,7 @@ using ThreadWeaver::Queue;
 
 namespace Plasma
 {
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 72) && KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
 void warnAboutDeprecatedMetaData(const KPluginInfo &pluginInfo)
 {
     if (!pluginInfo.libraryPath().isEmpty()) {
@@ -61,6 +68,7 @@ public:
         : q(parent)
     {
         matchChangeTimer.setSingleShot(true);
+        matchChangeTimer.setTimerType(Qt::TimerType::PreciseTimer); // Without this, autotest will fail due to imprecision of this timer
         delayTimer.setSingleShot(true);
 
         QObject::connect(&matchChangeTimer, &QTimer::timeout, q, [this]() {
@@ -82,11 +90,28 @@ public:
 
     void scheduleMatchesChanged()
     {
-        if (lastMatchChangeSignalled.hasExpired(250)) {
+        // We avoid over-refreshing the client. We only refresh every this much milliseconds
+        constexpr int refreshPeriod = 250;
+        // This will tell us if we are reseting the matches to start a new search. RunnerContext::reset() clears its query string for its emission
+        if (context.query().isEmpty()) {
+            matchChangeTimer.stop();
+            // This actually contains the query string for the new search that we're launching (if any):
+            if (!this->untrimmedTerm.trimmed().isEmpty()) {
+                // We are starting a new search, we shall stall for some time before deciding to show an empty matches list.
+                // This stall should be enough for the engine to provide more meaningful result, so we avoid refreshing with
+                // an empty results list if possible.
+                matchChangeTimer.start(refreshPeriod);
+                // We "pretend" that we have refreshed it so the next call will be forced to wait the timeout:
+                lastMatchChangeSignalled.restart();
+            } else {
+                // We have an empty input string, so it's not a real query. We don't expect any results to come, so no need to stall
+                Q_EMIT q->matchesChanged(context.matches());
+            }
+        } else if (lastMatchChangeSignalled.hasExpired(refreshPeriod)) {
             matchChangeTimer.stop();
             Q_EMIT q->matchesChanged(context.matches());
         } else {
-            matchChangeTimer.start(250 - lastMatchChangeSignalled.elapsed());
+            matchChangeTimer.start(refreshPeriod - lastMatchChangeSignalled.elapsed());
         }
     }
 
@@ -313,13 +338,17 @@ public:
         searchJobs.remove(runJob);
         oldSearchJobs.remove(runJob);
 
-        if (searchJobs.isEmpty() && context.matches().isEmpty()) {
-            // we finished our run, and there are no valid matches, and so no
-            // signal will have been sent out. so we need to emit the signal
-            // ourselves here
-            Q_EMIT q->matchesChanged(context.matches());
-        }
         if (searchJobs.isEmpty()) {
+            // If there are any new matches scheduled to be notified, we should anticipate it and just refresh right now
+            if (matchChangeTimer.isActive()) {
+                matchChangeTimer.stop();
+                Q_EMIT q->matchesChanged(context.matches());
+            } else if (context.matches().isEmpty()) {
+                // we finished our run, and there are no valid matches, and so no
+                // signal will have been sent out. so we need to emit the signal
+                // ourselves here
+                Q_EMIT q->matchesChanged(context.matches());
+            }
             Q_EMIT q->queryFinished();
         }
     }
@@ -731,8 +760,8 @@ void RunnerManager::run(const QueryMatch &match)
 
 bool RunnerManager::runMatch(const QueryMatch &match)
 {
-    d->addToHistory();
     if (match.type() == Plasma::QueryMatch::InformationalMatch && !match.selectedAction()) {
+        d->addToHistory();
         const QString info = match.data().toString();
         if (!info.isEmpty()) {
             Q_EMIT setSearchTerm(info, info.length());
@@ -740,7 +769,15 @@ bool RunnerManager::runMatch(const QueryMatch &match)
         }
     }
     d->context.run(match);
-    return true;
+    if (!d->context.shouldIgnoreCurrentMatchForHistory()) {
+        d->addToHistory();
+    }
+    if (d->context.requestedQueryString().isEmpty()) {
+        return true;
+    } else {
+        Q_EMIT setSearchTerm(d->context.requestedQueryString(), d->context.requestedCursorPosition());
+        return false;
+    }
 }
 
 QList<QAction *> RunnerManager::actionsForMatch(const QueryMatch &match)
@@ -784,7 +821,7 @@ QVector<KPluginMetaData> RunnerManager::runnerMetaDataList(const QString &parent
         QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("krunner/dbusplugins"), QStandardPaths::LocateDirectory);
     const QStringList dbusRunnerFiles = KFileUtils::findAllUniqueFiles(dBusPlugindirs, QStringList(QStringLiteral("*.desktop")));
     for (const QString &dbusRunnerFile : dbusRunnerFiles) {
-        KPluginMetaData pluginMetaData = KPluginMetaData::fromDesktopFile(dbusRunnerFile, QStringList(QStringLiteral("plasma-runner.desktop")));
+        KPluginMetaData pluginMetaData = parseMetaDataFromDesktopFile(dbusRunnerFile);
         if (pluginMetaData.isValid() && !knownRunnerIds.contains(pluginMetaData.pluginId())) {
             pluginMetaDatas.append(pluginMetaData);
             knownRunnerIds.insert(pluginMetaData.pluginId());
@@ -796,10 +833,9 @@ QVector<KPluginMetaData> RunnerManager::runnerMetaDataList(const QString &parent
     const QString constraint = parentApp.isEmpty() ? QStringLiteral("not exist [X-KDE-ParentApp] or [X-KDE-ParentApp] == ''")
                                                    : QStringLiteral("[X-KDE-ParentApp] == '") + parentApp + QLatin1Char('\'');
 
-    const KService::List offers = KServiceTypeTrader::self()->query(QStringLiteral("Plasma/Runner"), constraint);
     QT_WARNING_PUSH
-    QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
-    QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
+    QT_WARNING_DISABLE_DEPRECATED
+    const KService::List offers = KServiceTypeTrader::self()->query(QStringLiteral("Plasma/Runner"), constraint);
     const KPluginInfo::List backwardCompatPluginInfos = KPluginInfo::fromServices(offers);
     QT_WARNING_POP
 
@@ -817,7 +853,7 @@ QVector<KPluginMetaData> RunnerManager::runnerMetaDataList(const QString &parent
 
 QVector<KPluginMetaData> RunnerManager::runnerMetaDataList()
 {
-    QVector<KPluginMetaData> pluginMetaDatas = KPluginMetaData::findPlugins(QStringLiteral("kf5/krunner"));
+    QVector<KPluginMetaData> pluginMetaDatas = KPluginMetaData::findPlugins(QStringLiteral("kf" QT_STRINGIFY(QT_VERSION_MAJOR) "/krunner"));
     QSet<QString> knownRunnerIds;
     knownRunnerIds.reserve(pluginMetaDatas.size());
     for (const KPluginMetaData &pluginMetaData : std::as_const(pluginMetaDatas)) {
@@ -828,19 +864,18 @@ QVector<KPluginMetaData> RunnerManager::runnerMetaDataList()
         QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("krunner/dbusplugins"), QStandardPaths::LocateDirectory);
     const QStringList dbusRunnerFiles = KFileUtils::findAllUniqueFiles(dBusPlugindirs, QStringList(QStringLiteral("*.desktop")));
     for (const QString &dbusRunnerFile : dbusRunnerFiles) {
-        KPluginMetaData pluginMetaData = KPluginMetaData::fromDesktopFile(dbusRunnerFile, QStringList(QStringLiteral("plasma-runner.desktop")));
+        KPluginMetaData pluginMetaData = parseMetaDataFromDesktopFile(dbusRunnerFile);
         if (pluginMetaData.isValid() && !knownRunnerIds.contains(pluginMetaData.pluginId())) {
             pluginMetaDatas.append(pluginMetaData);
             knownRunnerIds.insert(pluginMetaData.pluginId());
         }
     }
 
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 72) && KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
     // also search for deprecated kservice-based KRunner plugins metadata
-    const KService::List offers = KServiceTypeTrader::self()->query(QStringLiteral("Plasma/Runner"));
     QT_WARNING_PUSH
-    QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
-    QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
+    QT_WARNING_DISABLE_DEPRECATED
+    const KService::List offers = KServiceTypeTrader::self()->query(QStringLiteral("Plasma/Runner"));
     const KPluginInfo::List backwardCompatPluginInfos = KPluginInfo::fromServices(offers);
     QT_WARNING_POP
 
@@ -858,7 +893,10 @@ QVector<KPluginMetaData> RunnerManager::runnerMetaDataList()
 #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 72)
 KPluginInfo::List RunnerManager::listRunnerInfo(const QString &parentApp)
 {
+    QT_WARNING_PUSH
+    QT_WARNING_DISABLE_DEPRECATED
     return KPluginInfo::fromMetaData(runnerMetaDataList(parentApp));
+    QT_WARNING_POP
 }
 #endif
 
@@ -1029,6 +1067,11 @@ void RunnerManager::reset()
     Q_EMIT queryFinished();
 }
 
+KPluginMetaData RunnerManager::convertDBusRunnerToJson(const QString &filename) const
+{
+    return parseMetaDataFromDesktopFile(filename);
+}
+
 void RunnerManager::enableKNotifyPluginWatcher()
 {
     if (!d->watcher) {
